﻿using System;
using System.Collections.Generic;
using System.Drawing;
using FooEditEngine.Windows.Properties;

namespace FooEditEngine.Windows
{
    class PrintableTextRender : IPrintableTextRender
    {
        StringFormat sf;
        Font font;
        Graphics g;

        public PrintableTextRender(Font font, Graphics g)
        {
            this.font = font;
            this.sf = StringFormat.GenericTypographic;
            this.sf.FormatFlags |= StringFormatFlags.MeasureTrailingSpaces;
            this.g = g;
            this.TokenToColor = new ColorTable();
        }

        public event ChangedRenderResourceEventHandler ChangedRenderResource;

        public void BeginDraw(Graphics g)
        {
            this.g = g;
        }

        public void EndDraw()
        {
        }

        public float HeaderHeight { get { return this.font.Height; } }

        public float FooterHeight { get { return this.font.Height; } }

        public double LineNemberWidth
        {
            get
            {
                int length = Int32.Parse(Resources.LineNumberFiledLength);
                if (length < 0)
                    throw new InvalidOperationException("LineNumberFiledLengthは正の値を指定してください");
                length++;   //余白を確保する
                SizeF metrics = g.MeasureString("0", this.font, Int16.MaxValue, this.sf);
                return metrics.Width * length;
            }
        }

        public Rectangle ClipRect
        {
            get;
            set;
        }

        public void DrawCachedBitmap(Rectangle rect)
        {
        }

        public void CacheContent()
        {
        }

        public bool IsVaildCache()
        {
            return false;
        }

        public void DrawLineNumber(int lineNumber, double x, double y)
        {
            string lineNumberFormat = "{0," + Resources.LineNumberFiledLength + "}";
            g.DrawString(string.Format(lineNumberFormat, lineNumber), this.font, new SolidBrush(this.TokenToColor.Fore), new PointF((float)x, (float)y),this.sf);
        }

        public void DrawLine(Point from,Point to)
        {
        }

        public void DrawCaret(Rectangle rect)
        {
        }

        public void DrawString(string str, double x, double y, StringAlignment align)
        {
            System.Drawing.StringAlignment old = this.sf.Alignment;
            
            this.sf.Alignment = System.Drawing.StringAlignment.Center;

            g.DrawString(str, this.font, new SolidBrush(this.TokenToColor.Fore), new RectangleF((float)x, (float)y, (float)this.ClipRect.Width, (float)this.font.Height), this.sf);
            
            this.sf.Alignment = old;
        }

        public void DrawOneLine(LineToIndexTable lti, int row, double x, double y, IEnumerable<Selection> SelectRanges, IEnumerable<Marker> MarkerRanges)
        {
            g.DrawString(lti[row], this.font, new SolidBrush(this.TokenToColor.Fore), new PointF((float)x, (float)y), this.sf);
        }

        public void FillBackground(Rectangle rect)
        {
        }

        public int GetIndexFromX(string str, double x)
        {
            return 0;
        }

        public double GetWidthFromIndex(string str, int index)
        {
            return 0;
        }

        public double GetWidth(string str)
        {
            return 0;
        }

        public double GetHeight(string str)
        {
            return this.font.Height;
        }

        public int AlignIndexToNearestCluster(string str, int index,AlignDirection flow)
        {
            return 0;
        }

        public List<LineToIndexTableData> BreakLine(Document doc,int startIndex,int endIndex,double maxwidth)
        {
            List<LineToIndexTableData> output = new List<LineToIndexTableData>();

            foreach (string str in Util.GetLines(doc, startIndex, endIndex))
            {
                LineToIndexTableData info = new LineToIndexTableData();

                if (str.Length == 0)
                {
                    info.index = startIndex;
                    info.length = 0;
                    output.Add(info);
                    return output;
                }

                int fitlen, index = 0, x = 0, width;
                do
                {
                    int linesFilled;
                    SizeF metrics = g.MeasureString(str.Substring(index), this.font, new SizeF((float)maxwidth, this.font.Height + 1), this.sf, out fitlen, out linesFilled);

                    x += width = (int)metrics.Width;

                    info.index = index + startIndex;
                    info.length = fitlen;
                    output.Add(info);

                    info = new LineToIndexTableData();

                    index += fitlen;
                } while (index < str.Length);

                output[output.Count - 1].lineend = true;

                startIndex += str.Length;
            }

            return output;
        }

        public double GetXFromIndex(string str, int index)
        {
            return 0;
        }

        public ColorTable TokenToColor
        {
            get;
            set;
        }

        public int TabWidthChar
        {
            get { float taboffset; return (int)this.sf.GetTabStops(out taboffset)[0]; }
            set { this.sf.SetTabStops(0,new float[]{value});}
        }

    }
}
